@@ -2,12 +2,13 @@ package WebService::PayPal::NVP;
 
 use Moo;
 use DateTime;
+use Encode qw( decode );
 use LWP::UserAgent ();
 use MooX::Types::MooseLike::Base qw( InstanceOf );
 use URI::Escape qw/uri_escape uri_escape_utf8 uri_unescape/;
 use WebService::PayPal::NVP::Response;
 
-our $VERSION = '0.004';
+our $VERSION = '0.005';
 
 has 'errors' => (
     is => 'rw',
@@ -55,9 +56,6 @@ sub _build_ua {
 sub _do_request {
     my ($self, $args) = @_;
 
-    my $req = HTTP::Request->new(POST => $self->url);
-    $req->content_type('application/x-www-form-urlencoded');
-
     my $authargs = {
         user      => $self->user,
         pwd       => $self->pwd,
@@ -68,20 +66,24 @@ sub _do_request {
 
     my $allargs = { %$authargs, %$args };
     my $content = $self->_build_content( $allargs );
-    $req->content($content);
-    my $res = $self->ua->request($req);
+    my $res = $self->ua->post(
+        $self->url,
+        'Content-Type' => 'application/x-www-form-urlencoded',
+        Content        => $content,
+    );
 
     unless ($res->code == 200) {
         $self->errors(["Failure: " . $res->code . ": " . $res->message]);
         return;
     }
 
-    my $resp = { map { uri_unescape($_) }
+    my $resp = { map { decode( 'UTF-8', uri_unescape($_) ) }
         map { split '=', $_, 2 }
             split '&', $res->content };
 
     my $res_object = WebService::PayPal::NVP::Response->new(
-        branch => $self->branch
+        branch => $self->branch,
+        raw => $resp
     );
 ;
     if ($resp->{ACK} ne 'Success') {
@@ -109,7 +111,7 @@ sub _do_request {
                 if ($val =~ /(\d+)-(\d+)-(\d+)T(\d+):(\d+):(\d+)Z/) {
                     my ($day, $month, $year, $hour, $min, $sec)
                      = ($3, $2, $1, $4, $5, $6);
-                    
+
                     $val = DateTime->new(
                         year    => $year,
                         month   => $month,
@@ -125,7 +127,7 @@ sub _do_request {
             };
         }
     }
-    return $res_object; 
+    return $res_object;
 }
 
 sub _build_content {
@@ -181,6 +183,12 @@ sub get_recurring_payments_profile_details {
     $self->_do_request($args);
 }
 
+sub get_transaction_details {
+    my ($self, $args) = @_;
+    $args->{method} = 'GetTransactionDetails';
+    $self->_do_request($args);
+}
+
 sub manage_recurring_payments_profile_status {
     my ($self, $args) = @_;
     $args->{method} = 'ManageRecurringPaymentsProfileStatus';
@@ -199,6 +207,12 @@ sub refund_transaction {
     $self->_do_request($args);
 }
 
+sub transaction_search {
+    my ( $self, $args ) = @_;
+    $args->{method} = 'TransactionSearch';
+    $self->_do_request($args);
+}
+
 1;
 __END__
 
@@ -220,7 +234,7 @@ Another difference with this module compared to Business::PayPal::NVP is that th
         sig    => 'xxxxxxx',
         branch => 'sandbox',
     );
-    
+
     my $res = $nvp->set_express_checkout({
         DESC              => 'Payment for something cool',
         AMT               => 25.00,
@@ -231,14 +245,14 @@ Another difference with this module compared to Business::PayPal::NVP is that th
         LANDINGPAGE       => 'Login',
         ADDOVERRIDE       => 1,
         SHIPTONAME        => "Customer Name",
-        SHIPTOSTREET      => "7 Customer Street", 
-        SHIPTOSTREET2     => "", 
-        SHIPTOCITY        => "Town", 
-        SHIPTOZIP         => "Postcode", 
-        SHIPTOEMAIL       => "customer\@example.com", 
+        SHIPTOSTREET      => "7 Customer Street",
+        SHIPTOSTREET2     => "",
+        SHIPTOCITY        => "Town",
+        SHIPTOZIP         => "Postcode",
+        SHIPTOEMAIL       => "customer\@example.com",
         SHIPTOCOUNTRYCODE => 'GB',
     });
-    
+
     if ($res->success) {
         # timestamps turned into DateTime objects
         say "Response received at "
@@ -254,7 +268,7 @@ Another difference with this module compared to Business::PayPal::NVP is that th
         }
 
         # get a redirect uri to paypal express checkout
-        # the Response object will automatically detect if you have 
+        # the Response object will automatically detect if you have
         # live or sandbox and return the appropriate url for you
         if (my $redirect_user_to = $res->express_checkout_uri) {
             $web_framework->redirect( $redirect_user_to );
@@ -290,6 +304,8 @@ Returns true if C<errors()> is non-empty.
 
 =head2 get_recurring_payments_profile_details( $HashRef )
 
+=head2 get_transaction_details( $HashRef )
+
 =head2 manage_recurring_payments_profile_status( $HashRef )
 
 =head2 mass_pay( $HashRef )
@@ -298,6 +314,8 @@ Returns true if C<errors()> is non-empty.
 
 =head2 set_express_checkout( $HashRef )
 
+=head2 transaction_search( $HashRef )
+
 =head2 ua( LWP::UserAgent->new( ... ) )
 
 This method allows you to provide your own UserAgent.  This object must be of
@@ -326,7 +344,7 @@ Brad Haywood <brad@geeksware.com>
 
 =head1 CREDITS
 
-A lot of this module was taken from L<Business::PayPal::NVP> by Scott Wiersdorf. 
+A lot of this module was taken from L<Business::PayPal::NVP> by Scott Wiersdorf.
 It was only rewritten in order to work properly in L<Catalyst::Model::Adaptor>.
 
 =head2 THANKS
